import { BlockchainIncludingTestnet, UserIdentifierParams as UserIdentifierParams$1, EVMBlockchainIncludingTestnet } from '@crossmint/common-sdk-base';
export { EVMBlockchainIncludingTestnet as Blockchain, EVMBlockchainIncludingTestnet, blockchainToChainId } from '@crossmint/common-sdk-base';
import { EntryPointVersion } from 'permissionless/_types/types';
import { AuthSig } from '@lit-protocol/types';
import { TORUS_NETWORK_TYPE } from '@web3auth/single-factor-auth';
import * as _zerodev_sdk from '@zerodev/sdk';
import { createKernelAccountClient, KernelSmartAccount } from '@zerodev/sdk';
import { LocalAccount, PublicClient, EIP1193Provider, HttpTransport, TypedDataDefinition, Hash, Hex } from 'viem';
import { BigNumber, BigNumberish } from 'ethers';
import * as permissionless_clients_pimlico from 'permissionless/clients/pimlico';
import { Deferrable } from '@ethersproject/properties';
import { TransactionRequest } from '@ethersproject/providers';
import { KernelValidator } from '@zerodev/ecdsa-validator';
import { EntryPoint } from 'permissionless/types/entrypoint';
import { Chain } from 'viem/chains';
import * as _ethersproject_transactions from '@ethersproject/transactions';
import { BytesLike } from 'ethers/lib/utils';
import { ReservoirWallet } from '@reservoir0x/reservoir-sdk';

type CrossmintAASDKInitParams = {
    apiKey: string;
};
type PasskeysSDKInitParams = {
    apiKey: string;
};
type UserIdentifierParams = {
    email?: string;
    userId?: string;
    phoneNumber?: string;
};
type UserIdentifier = {
    type: "whiteLabel";
    userId: string;
} | {
    type: "email";
    email: string;
} | {
    type: "phoneNumber";
    phoneNumber: string;
};
type SignerMap = {
    viem: Client;
};
type SignerType = keyof SignerMap;
type Web3AuthSigner = {
    type: "WEB3_AUTH";
    clientId: string;
    verifierId: string;
    web3AuthNetwork: TORUS_NETWORK_TYPE;
    jwt: string;
};
type FireblocksNCWSignerBase = {
    type: "FIREBLOCKS_NCW";
    passphrase: string;
};
type FireblocksNCWSigner = FireblocksNCWSignerBase | (FireblocksNCWSignerBase & {
    walletId: string;
    deviceId: string;
});
type ViemAccount = {
    type: "VIEM_ACCOUNT";
    account: LocalAccount;
};
type Signer = EIP1193Provider | Web3AuthSigner | ViemAccount;
interface WalletConfig {
    signer: Signer;
}
interface PasskeyCipher {
    chain: BlockchainIncludingTestnet;
    walletAddress: string;
    cipher: Cipher;
}
interface LitProtocolCipherData {
    pkpPublicKey?: string;
    pkpEthAddress?: string;
    cipherText?: string;
    dataToEncryptHash?: string;
}
type EncryptInput = {
    messageToEncrypt: string;
    pkpPublicKey: string;
    pkpEthAddress: string;
    capacityDelegationAuthSig: AuthSig;
};
type DecryptInput = {
    pkpPublicKey: string;
    pkpEthAddress: string;
    cipherText: string;
    dataToEncryptHash: string;
    capacityDelegationAuthSig: AuthSig;
};
type Cipher = {
    method: "lit_protocol";
    data: LitProtocolCipherData;
};
type BackwardsCompatibleChains = "goerli";
type Client = {
    publicClient: PublicClient;
    walletClient: ReturnType<typeof createKernelAccountClient<EntryPoint, HttpTransport, TChain, KernelSmartAccount<EntryPoint, HttpTransport, TChain>>>;
};

type StoreAbstractWalletInput = {
    userIdentifier: UserIdentifier;
    type: string;
    smartContractWalletAddress: string;
    eoaAddress: string;
    sessionKeySignerAddress: string;
    version: number;
    baseLayer: string;
    chainId: number;
    entryPointVersion: EntryPointVersion;
};
type GenerateSignatureDataInput = {
    smartContractWalletAddress: `0x${string}`;
    sessionKeyData?: string;
    killSwitchData?: string;
    chain: BlockchainIncludingTestnet | "evm";
    version: number;
};
type TransferInput = {
    tokenId: number;
    chain: string;
    fromAddress: string;
    toAddress: string;
    tokenMintAddress: string;
};

declare class LoggerWrapper {
    private extraInfo;
    private logIdempotencyKey;
    constructor(className: string, extraInfo?: {}, logIdempotencyKey?: string);
    private logInput;
    private logOutput;
    private logError;
    protected logPerformance<T>(name: string, cb: () => Promise<T>): Promise<T>;
}

declare abstract class BaseCrossmintService extends LoggerWrapper {
    protected crossmintAPIHeaders: Record<string, string>;
    protected crossmintBaseUrl: string;
    private static urlMap;
    constructor(apiKey: string);
    protected fetchCrossmintAPI(endpoint: string, options: {
        body?: string | undefined;
        method: string;
    } | undefined, onServerErrorMessage: string): Promise<any>;
    protected getUrlFromEnv(environment: string): string;
}

declare class CrossmintWalletService extends BaseCrossmintService {
    createSessionKey(address: string | `0x${string}`): Promise<any>;
    storeAbstractWallet(input: StoreAbstractWalletInput): Promise<any>;
    getAbstractWalletEntryPointVersion(userIdentifier: UserIdentifierParams$1, chain: EVMBlockchainIncludingTestnet): Promise<any>;
    generateChainData(input: GenerateSignatureDataInput): Promise<any>;
    getOrAssignWallet(userIdentifier: UserIdentifier): Promise<any>;
    unassignWallet(userIdentifier: UserIdentifier): Promise<any>;
    rpc(walletId: string, deviceId: string, payload: string): Promise<any>;
    createTransaction(data: any, walletId: string, assetId: string, typedMessage: boolean): Promise<any>;
    getSignature(txId: string): Promise<any>;
    getAddress(walletId: string, accountId: number, assetId: string): Promise<any>;
    getNCWIdentifier(deviceId: string): Promise<any>;
    checkVersion(address: string): Promise<any>;
    updateWallet(address: string, enableSig: string, version: number): Promise<any>;
    fetchNFTs(address: string, chain: EVMBlockchainIncludingTestnet): Promise<any>;
}

interface EVMToken {
    chain: EVMBlockchainIncludingTestnet;
    contractAddress: string;
}
interface NFTEVMToken extends EVMToken {
    tokenId: string;
    type: "nft";
}
interface SFTEVMToken extends EVMToken {
    tokenId: string;
    type: "sft";
}
interface ERC2OEVMToken extends EVMToken {
    type: "ft";
}
interface SolanaToken {
    mintHash: string;
    chain: "solana";
    type: "nft";
}
interface CardanoToken {
    chain: "cardano";
    assetId: string;
    type: "nft";
}
declare function isEVMToken(value: unknown): value is EVMToken;
type Token = EVMToken | SolanaToken | CardanoToken;
type TokenType = "nft" | "sft" | "ft";
type ERC20TransferType = {
    token: ERC2OEVMToken;
    amount: BigNumber;
};
type SFTTransferType = {
    token: SFTEVMToken;
    quantity: number;
};
type NFTTransferType = {
    token: NFTEVMToken | SolanaToken | CardanoToken;
};
type TransferType = ERC20TransferType | SFTTransferType | NFTTransferType;

declare function getFireblocksAssetId(chain: EVMBlockchainIncludingTestnet): string;
declare const getUrlProviderByBlockchain: (this: any, ...args: any[]) => any;
declare const getBlockExplorerByBlockchain: (this: any, ...args: any[]) => any;
declare function getTickerByBlockchain(chain: EVMBlockchainIncludingTestnet): string;
declare function getTickerNameByBlockchain(chain: EVMBlockchainIncludingTestnet): string;
declare const getZeroDevProjectIdByBlockchain: (this: any, ...args: any[]) => any;
declare const getViemNetwork: (this: any, ...args: any[]) => any;
declare const getBundlerRPC: (this: any, ...args: any[]) => any;
declare const getPaymasterRPC: (this: any, ...args: any[]) => any;
type entryPoint = EntryPoint;
type TChain = Chain | undefined;

interface ExternalCustodian {
    type: "EXTERNAL";
    address: string;
}
type Custodian = ExternalCustodian | "CROSSMINT_MANAGED" | "NONE";

type GasFeeTransactionParams = {
    maxFeePerGas?: BigNumberish;
    maxPriorityFeePerGas?: BigNumberish;
};
declare class EVMAAWallet<B extends EVMBlockchainIncludingTestnet = EVMBlockchainIncludingTestnet> extends LoggerWrapper {
    private sessionKeySignerAddress?;
    private crossmintService;
    private publicClient;
    private ecdsaValidator;
    private account;
    private kernelClient;
    private entryPoint;
    chain: B;
    constructor(account: KernelSmartAccount<EntryPoint, HttpTransport, TChain>, crossmintService: CrossmintWalletService, chain: B, publicClient: PublicClient, ecdsaValidator: KernelValidator<entryPoint, "ECDSAValidator">, entryPoint: EntryPoint);
    getPaymasterClient(): _zerodev_sdk.ZeroDevPaymasterClient<EntryPoint> | permissionless_clients_pimlico.PimlicoPaymasterClient<EntryPoint>;
    getAddress(): `0x${string}`;
    signMessage(message: string | Uint8Array): Promise<`0x${string}`>;
    signTypedData(params: TypedDataDefinition): Promise<`0x${string}`>;
    sendTransaction(transaction: Deferrable<TransactionRequest>): Promise<Hash>;
    transfer(toAddress: string, config: TransferType): Promise<string>;
    getSigner<Type extends SignerType>(type: Type): SignerMap[Type];
    setSessionKeySignerAddress(sessionKeySignerAddress: Hex): void;
    setCustodianForTokens(tokenType?: TokenType, custodian?: Custodian): Promise<void>;
    upgradeVersion(): Promise<void>;
    getNFTs(): Promise<any>;
    getLegacyTransactionFeesParamsIfApply(gasFeeParams?: GasFeeTransactionParams): {
        maxPriorityFeePerGas?: bigint | undefined;
        maxFeePerGas?: bigint | undefined;
    } | {
        maxFeePerGas: any;
        maxPriorityFeePerGas: any;
    };
}

declare const parseToken: (token: any) => any;

declare const ZERO_DEV_TYPE = "ZeroDev";
declare const CURRENT_VERSION = 0;
declare const DATADOG_CLIENT_TOKEN = "pub035be8a594b35be1887b6ba76c4029ca";
declare const CROSSMINT_DEV_URL = "http://localhost:3000/api";
declare const CROSSMINT_STG_URL = "https://staging.crossmint.com/api";
declare const CROSSMINT_PROD_URL = "https://www.crossmint.com/api";
declare const ZD_ETHEREUM_PROJECT_ID = "9ee29857-8077-404b-9a9a-31eeea996a4a";
declare const ZD_POLYGON_PROJECT_ID = "023d4a21-d801-4450-b629-24439ab1369d";
declare const ZD_BSC_PROJECT_ID = "3d166617-da86-494b-9348-e8a13343bc04";
declare const ZD_OPTIMISM_PROJECT_ID = "e9314f9e-a13d-414f-b965-c591a0248243";
declare const ZD_ARBITRUM_PROJECT_ID = "1641cd99-c1ef-404a-9d26-a9dc67b1ba51";
declare const ZD_GOERLI_PROJECT_ID = "3cfecfb6-9d7d-4ef6-acaa-ac8f79f6cd5a";
declare const ZD_SEPOLIA_PROJECT_ID = "7ff22858-06f0-4f3a-8b46-5b41d8c75d0e";
declare const ZD_AMOY_PROJECT_ID = "3b24773b-d91e-4c01-8ce5-04807463bbca";
declare const ZD_ZKATANA_PROJECT_ID = "d54706a0-304b-419e-8a33-03c26ba3f0e9";
declare const ZD_ZKYOTO_PROJECT_ID = "ce986d52-4f27-4a2b-b429-eb2322f15f32";
declare const ZD_ASTAR_PROJECT_ID = "6204f336-643f-41af-94e1-f8d146c91675";
declare const ZD_BASE_SEPOLIA_PROJECT_ID = "3eb830c5-f91b-48e0-bb7d-dc30103a60b2";
declare const ZD_OPTIMISM_SEPOLIA_PROJECT_ID = "6188b92b-993e-4f39-be22-56e4806416a8";
declare const ZD_ARBITRUM_SEPOLIA_PROJECT_ID = "8b8b6d1a-184c-4198-8f29-c07e63aad595";
declare const ZD_BASE_PROJECT_ID = "5535aa3b-4f9c-45af-9c38-0072369564a3";
declare const ZD_ARBITRUM_NOVA_PROJECT_ID = "b6a6db7c-65de-4f74-9d8f-f70d9c083531";
declare const RELAY_API_KEY = "c8999b7e-8bb7-416d-a854-dac8632f9ee6_crossmint";
declare const SCW_SERVICE = "SDK_AA";
declare const BUNDLER_RPC = "https://rpc.zerodev.app/api/v2/bundler/";
declare const PAYMASTER_RPC = "https://rpc.zerodev.app/api/v2/paymaster/";
declare const PM_BASE_SEPOLIA_RPC = "https://api.developer.coinbase.com/rpc/v1/base-sepolia/6BTAmOQZ0x1YWtI24hIKKqdHWmVP1UXf";
declare const PM_BASE_RPC = "https://api.developer.coinbase.com/rpc/v1/base/6BTAmOQZ0x1YWtI24hIKKqdHWmVP1UXf";

declare class NotAuthorizedError extends Error {
    code: string;
    constructor(message: string);
}
declare class RateLimitError extends Error {
    code: string;
    retryAfterMs: number;
    constructor(message: string, retryAfterMs: number);
}
declare class PassphraseRequiredError extends Error {
    code: string;
    constructor(message: string);
}
declare class KeysGenerationError extends Error {
    code: string;
    constructor(message: string);
}
declare class SignTransactionError extends Error {
    code: string;
    constructor(message: string);
}
declare class TransferError extends Error {
    code: string;
    constructor(message: string);
}
declare class TransactionError extends Error {
    code: string;
    constructor(message: string);
}
declare class CrossmintServiceError extends Error {
    code: string;
    constructor(message: string);
}
/**
 * Generic undefined error
 */
declare class NonCustodialWalletError extends Error {
    code: string;
    constructor(message: string);
}
/**
 * Generic undefined error
 */
declare class WalletSdkError extends Error {
    code: string;
    constructor(message: string);
}
declare class PasskeySdkError extends Error {
    code: string;
    constructor(message: string);
}
declare class LitProtocolError extends Error {
    code: string;
    constructor(message: string);
}
declare function errorToJSON(error: Error | unknown): any;

declare const createOwnerSigner: (this: any, ...args: any[]) => any;
declare function isAccount(signer: any): signer is ViemAccount;

declare function isLocalhost(): boolean;
declare function isEmpty(str: string | undefined | null): str is undefined | null;
declare function transformBackwardsCompatibleChains<B extends BlockchainIncludingTestnet = BlockchainIncludingTestnet>(chain: B | BackwardsCompatibleChains): B;

type VerifyMessageParams = {
    address: `0x${string}`;
    message: string;
    signature: `0x${string}` | Uint8Array;
    chain: EVMBlockchainIncludingTestnet;
};
/**
 * We need to support odd value and data fields in a tx, as that data comes from WC
 * ZeroDev implements hexlify, but doesn't support to send an odd string in value or data fields
 * so we need to make the hexlify fix manually when sending a transaction.
 * That fix can be found here:
 * https://github.com/ethers-io/ethers.js/commit/a12030ad29aa13c02aa75d9e0860f4986a0043b4#diff-047e7ebfdbd5c41e762cda03593bd15ed8b3121dece67262729dfcbde7040818R221
 * And more context on this issue:
 * https://github.com/ethers-io/ethers.js/issues/614
 *
 * Applying to it hexValue is used as a workaround on Crossbit, but doesn't cover all the cases (i.e. 0x71afd498d0000).
 * This function is the same as hexValue but without hexStripZeros, which is what makes it to not work.
 */
declare function decorateSendTransactionData(transaction: Deferrable<TransactionRequest>): Promise<{
    to?: string | Promise<string | undefined> | undefined;
    from?: string | Promise<string | undefined> | undefined;
    nonce?: BigNumberish | Promise<BigNumberish | undefined> | undefined;
    gasLimit?: BigNumberish | Promise<BigNumberish | undefined> | undefined;
    gasPrice?: BigNumberish | Promise<BigNumberish | undefined> | undefined;
    data?: BytesLike | Promise<BytesLike | undefined> | undefined;
    value?: BigNumberish | Promise<BigNumberish | undefined> | undefined;
    chainId?: number | Promise<number | undefined> | undefined;
    type?: number | Promise<number | undefined> | undefined;
    accessList?: _ethersproject_transactions.AccessListish | Promise<_ethersproject_transactions.AccessListish | undefined> | undefined;
    maxPriorityFeePerGas?: BigNumberish | Promise<BigNumberish | undefined> | undefined;
    maxFeePerGas?: BigNumberish | Promise<BigNumberish | undefined> | undefined;
    customData?: Record<string, any> | Promise<Record<string, any> | undefined> | undefined;
    ccipReadEnabled?: boolean | Promise<boolean | undefined> | undefined;
}>;
declare function getNonce(nonce: BigNumberish | Promise<BigNumberish | undefined> | undefined): Promise<number | undefined>;
declare function convertData(data: BytesLike | Promise<BytesLike | undefined> | undefined): Promise<`0x${string}` | undefined>;
declare function verifyMessage({ address, message, signature, chain }: VerifyMessageParams): Promise<boolean>;
declare function hasEIP1559Support(chain: EVMBlockchainIncludingTestnet): boolean;

declare class CrossmintAASDK extends LoggerWrapper {
    crossmintService: CrossmintWalletService;
    private constructor();
    static init(params: CrossmintAASDKInitParams): CrossmintAASDK;
    getOrCreateWallet<B extends BlockchainIncludingTestnet = BlockchainIncludingTestnet>(user: UserIdentifierParams$1, chain: B | BackwardsCompatibleChains, walletConfig: WalletConfig): Promise<EVMAAWallet<(B & "arbitrum-sepolia") | (B & "base-goerli") | (B & "base-sepolia") | (B & "bsc-testnet") | (B & "ethereum-goerli") | (B & "ethereum-sepolia") | (B & "polygon-mumbai") | (B & "polygon-amoy") | (B & "optimism-goerli") | (B & "optimism-sepolia") | (B & "zora-goerli") | (B & "zora-sepolia") | (B & "hypersonic-testnet") | (B & "ethereum") | (B & "polygon") | (B & "bsc") | (B & "optimism") | (B & "arbitrum") | (B & "base") | (B & "zora") | (B & "arbitrumnova") | (B & "zkatana") | (B & "zkyoto") | (B & "astar-zkevm") | (B & "apex")>>;
    /**
     * Clears all key material and state from device storage, related to all wallets stored. Call this method when the user signs out of your app, if you don't have a user identifier.
     */
    purgeAllWalletData(): Promise<void>;
    private getEntryPointVersion;
}

declare function reservoirAdapter(aaWallet: EVMAAWallet): ReservoirWallet;

declare class CrossmintPasskeyService extends BaseCrossmintService {
    getPasskeyCiphers(walletLocator: string): Promise<PasskeyCipher>;
    upsertPasskeyCiphers(walletLocator: string, passkeyCipher: any): Promise<any>;
    getCapacityCreditsOwnerSignature(): Promise<any>;
}

declare class LitService {
    private litNodeClient;
    private litAuthClient;
    connect(): Promise<void>;
    registerWithWebAuthn(identifier: string): Promise<{
        pkpEthAddress: string;
        pkpPublicKey: string;
    }>;
    encrypt({ messageToEncrypt, pkpPublicKey, pkpEthAddress, capacityDelegationAuthSig }: EncryptInput): Promise<{
        ciphertext: string;
        dataToEncryptHash: string;
    }>;
    decrypt({ pkpPublicKey, pkpEthAddress, cipherText, dataToEncryptHash, capacityDelegationAuthSig, }: DecryptInput): Promise<string>;
    private prepareLit;
}

declare class PasskeysSDK {
    crossmintService: CrossmintPasskeyService;
    litService: LitService;
    constructor(crossmintService: CrossmintPasskeyService, litService: LitService);
    static init(params: PasskeysSDKInitParams): PasskeysSDK;
    signUp(chain: BlockchainIncludingTestnet, walletAddress: string): Promise<{
        pkpEthAddress: string;
        pkpPublicKey: string;
    } | undefined>;
    encrypt(chain: BlockchainIncludingTestnet, walletAddress: string, secretToEncrypt: string): Promise<{
        ciphertext: string;
        dataToEncryptHash: string;
    } | undefined>;
    decrypt(chain: BlockchainIncludingTestnet, walletAddress: string): Promise<string | undefined>;
    private getPasskeyCiphers;
    private checkCiphersExist;
    private checkPPKAlreadyMinted;
    private checkSecretAlreadyCreated;
    private savePKP;
    private saveCypherText;
}

export { BUNDLER_RPC, type BackwardsCompatibleChains, CROSSMINT_DEV_URL, CROSSMINT_PROD_URL, CROSSMINT_STG_URL, CURRENT_VERSION, type CardanoToken, type Client, CrossmintAASDK, type CrossmintAASDKInitParams, CrossmintServiceError, CrossmintWalletService, type Custodian, DATADOG_CLIENT_TOKEN, type DecryptInput, type ERC20TransferType, type ERC2OEVMToken, EVMAAWallet, type EVMToken, type EncryptInput, type FireblocksNCWSigner, type GenerateSignatureDataInput, KeysGenerationError, type LitProtocolCipherData, LitProtocolError, type NFTEVMToken, type NFTTransferType, NonCustodialWalletError, NotAuthorizedError, PAYMASTER_RPC, PM_BASE_RPC, PM_BASE_SEPOLIA_RPC, type PasskeyCipher, PasskeySdkError, PasskeysSDK, type PasskeysSDKInitParams, PassphraseRequiredError, RELAY_API_KEY, RateLimitError, SCW_SERVICE, type SFTEVMToken, type SFTTransferType, SignTransactionError, type SignerMap, type SignerType, type SolanaToken, type StoreAbstractWalletInput, type TChain, type Token, type TokenType, TransactionError, TransferError, type TransferInput, type TransferType, type UserIdentifier, type UserIdentifierParams, type VerifyMessageParams, type ViemAccount, type WalletConfig, WalletSdkError, type Web3AuthSigner, ZD_AMOY_PROJECT_ID, ZD_ARBITRUM_NOVA_PROJECT_ID, ZD_ARBITRUM_PROJECT_ID, ZD_ARBITRUM_SEPOLIA_PROJECT_ID, ZD_ASTAR_PROJECT_ID, ZD_BASE_PROJECT_ID, ZD_BASE_SEPOLIA_PROJECT_ID, ZD_BSC_PROJECT_ID, ZD_ETHEREUM_PROJECT_ID, ZD_GOERLI_PROJECT_ID, ZD_OPTIMISM_PROJECT_ID, ZD_OPTIMISM_SEPOLIA_PROJECT_ID, ZD_POLYGON_PROJECT_ID, ZD_SEPOLIA_PROJECT_ID, ZD_ZKATANA_PROJECT_ID, ZD_ZKYOTO_PROJECT_ID, ZERO_DEV_TYPE, convertData, createOwnerSigner, decorateSendTransactionData, type entryPoint, errorToJSON, getBlockExplorerByBlockchain, getBundlerRPC, getFireblocksAssetId, getNonce, getPaymasterRPC, getTickerByBlockchain, getTickerNameByBlockchain, getUrlProviderByBlockchain, getViemNetwork, getZeroDevProjectIdByBlockchain, hasEIP1559Support, isAccount, isEVMToken, isEmpty, isLocalhost, parseToken, reservoirAdapter, transformBackwardsCompatibleChains, verifyMessage };
